#include <iostream>
#include <cmath>
#include <map>

using namespace std;

typedef map<double, double> polynom_map;

struct polynom {
  polynom_map monoms;
  double order;

  void cleanup() {
    order = 0;
    // determine order
    for (polynom_map::iterator i = monoms.begin(); i != monoms.end(); i++) {
      if (i->first > order) order = i->first;
    }
    // fill the gaps
    for (int j = 0; j <= order; j++) monoms[j] = monoms[j];
  }
};

struct polynom_div_result {
  polynom quotient;
  polynom remainder;

  void cleanup() {
    quotient.cleanup();
    remainder.cleanup();
  }
};

polynom_div_result calc(polynom nominator, polynom denominator) {

  polynom_div_result res;

  double r, o; // Koeffizienten-Quotient, aktueller Grad

  polynom_map::reverse_iterator i;
  polynom_map::const_iterator j;

  if (denominator.order <= nominator.order) {

    for (i = nominator.monoms.rbegin(); i != nominator.monoms.rend(); i++) {

      if (i->first < denominator.order) break;

      r = i->second / denominator.monoms[denominator.order];
      o = i->first - denominator.order;

      for (j = denominator.monoms.begin(); j != denominator.monoms.end(); j++) {
        nominator.monoms[j->first + o] -= r * denominator.monoms[j->first];
      }

      res.quotient.monoms[o] = r;
    }
  }

  res.remainder = nominator;
  res.cleanup();
  return res;
}


void print(polynom p) {
  for (polynom_map::const_reverse_iterator i = p.monoms.rbegin(); i != p.monoms.rend(); i++) {
    if (i->second == 0) continue;
    cout << (i == p.monoms.rbegin() && i->second > 0 ? "" : (i->second < 0 ? " - " : " + ")) << abs(i->second);
    if (i->first > 0) {
      cout << "x";
      if (i->first > 1) cout << '^' << i->first;
    }
  }
}

void main() {
  polynom nominator, denominator;
  polynom_div_result res;

  nominator.monoms[3] = 4;
  nominator.monoms[2] = -5;
  nominator.monoms[1] = 3;
  nominator.monoms[0] = -2;

  denominator.monoms[2] = 2;
  denominator.monoms[0] = -1;

  nominator.cleanup();
  denominator.cleanup();

  print(nominator);
  cout << " : ";
  print(denominator);

  res = calc(nominator, denominator);

  cout << '\n';

  print(res.quotient);
  cout << " remainder: ";
  print(res.remainder);
}